//
//  Image_PixelSPlugIn.m
//  Image PixelS2
//
//  Created by Matthias Oostrik on 14/Dec/10.
//  Copyleft 2010 Magdatt
//


#import "Image_PixelSPlugIn.h"

#define	kQCPlugIn_Name				@"Image PixelS2"
#define	kQCPlugIn_Description		@"Extracts from an image the Luminance (RGB), Red, Green, Blue or Alpha values of all pixels in generic RGB colorspace"

@implementation Image_PixelSPlugIn

@dynamic inputType, inputBoost, inputImage, outputValues;

+ (NSDictionary*) attributes
{
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/* Return the attributes for the plug-in property ports */
	if([key isEqualToString:@"inputImage"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Image", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputBoost"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Boost", QCPortAttributeNameKey,
				[NSNumber numberWithFloat:10.0], QCPortAttributeMaximumValueKey,
				[NSNumber numberWithFloat:0.0], QCPortAttributeMinimumValueKey, 
				[NSNumber numberWithFloat:1.0], QCPortAttributeDefaultValueKey,
				nil];
	if([key isEqualToString:@"inputType"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Type", QCPortAttributeNameKey,
				[NSNumber numberWithUnsignedInteger:4], QCPortAttributeMaximumValueKey, 
				[NSNumber numberWithUnsignedInteger:0], QCPortAttributeDefaultValueKey,
				[NSArray arrayWithObjects:@"Luminance (RGB)", @"Red", @"Green", @"Blue", @"Alpha", nil], QCPortAttributeMenuItemsKey,
				nil];
	if([key isEqualToString:@"outputValues"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Component Structure", QCPortAttributeNameKey, nil];
	
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	return kQCPlugInExecutionModeProcessor;
}


+ (QCPlugInTimeMode) timeMode
{
	return kQCPlugInTimeModeNone;
}

@end

@implementation Image_PixelSPlugIn (Execution)

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	float*				baseAddress;
	// Colorspace must be RGBLinear so that they are not transformed and negative values are preserved
	CGColorSpaceRef		colorSpace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
	NSUInteger			width, height, 
						floatsPerRow,
						x, y,
						inputSelect = self.inputType;
	float				mono, 
						b = self.inputBoost;
	NSRect				bounds = [self.inputImage imageBounds];
	CFMutableArrayRef	monoArray;
	CFNumberRef			monoNumber;
	
	// Downloading image
	[self.inputImage lockBufferRepresentationWithPixelFormat:QCPlugInPixelFormatRGBAf colorSpace:colorSpace forBounds:bounds];
	
	// Getting image info
	baseAddress = (float*)[self.inputImage bufferBaseAddress];
	width = [self.inputImage bufferPixelsWide];
	height = [self.inputImage bufferPixelsHigh];
	floatsPerRow = [self.inputImage bufferBytesPerRow]/sizeof(float);
	
	// Creating coordinate arrays
	monoArray = CFArrayCreateMutable(kCFAllocatorDefault, width*height, &kCFTypeArrayCallBacks);
	
	// We read each image pixel and store its R,G,B or Alpha value in the array
	if (inputSelect == 0) {
		for (y=0; y<height; ++y) {
			for (x=0; x<width; ++x) {
				mono = 0.299 * b * baseAddress[4*x+y*floatsPerRow] + 0.587 * b * baseAddress[4*x+1+y*floatsPerRow] + 0.114 * b * baseAddress[4*x+2+y*floatsPerRow];
				monoNumber = CFNumberCreate(kCFAllocatorDefault, kCFNumberFloatType, &mono);
				CFArrayAppendValue(monoArray, monoNumber);
				CFRelease(monoNumber);
			}
		}	
	}
	else {	
		for (y=0; y<height; ++y) {
			for (x=0; x<width; ++x) {
				mono = baseAddress[4*x+inputSelect-1+y*floatsPerRow];
				monoNumber = CFNumberCreate(kCFAllocatorDefault, kCFNumberFloatType, &mono);
				CFArrayAppendValue(monoArray, monoNumber);
				CFRelease(monoNumber);
			}
		}
	}
	
	[self.inputImage unlockBufferRepresentation];
	
	self.outputValues = (NSArray *) monoArray;
	
	CFRelease(colorSpace);
	CFRelease(monoArray);
	
	return YES;
}

@end
